#!/usr/bin/perl
# decode.pl 
# 
# ./decode.pl register_name register_value
# translate in human the values of init registers (fbiinit[1-7])
#
use strict;
use integer;


my ($reg_name, @vals) = @ARGV;

my @fbiinit0 = ( 
	[  0,	"VGA passthrough" ],
	[  1,	"FBI Graphic Reset <0>." ],
	[  2,	"FBI FIFO Reset <0>." ],
	[  3,	"Byte swizzle incoming register writes <0>." ],
	[  4,	"Stall PCI Enable for high watermark <1>." ],
	[  5,	"" ],
	[ 10,	"PCI FIFO empty entries low watermark <0x10>." ],
	[ 11,	"LFB access stored in memory FIFO <0>." ],
	[ 12,	"Texture memory access stored in memory FIFO <0>" ],
	[ 13,	"memory FIFO enable (Deafult 0)." ],
	[ 24,	"memory FIFO high watermark (<<5) <0>." ],
	[ 30,	"memory FIFO write burst high watermark <0>." ],
	[ 31,	"" ]
);

my @fbiinit1 = (
	[  0,	"PCI device funtion number (0=sst , 1=vga+sst)" ],
	[  1,	"Wait state for PCI write access (# of ws) <1>. *" ],
	[  2,	"SLI detected" ],
	[  3,	"enable LFB reads <0>." ],
	[  7,	"number of video tiles in X<0> 64x16 / 32x32[4:1]." ],
	[  8,	"Video Timing Reset <1>." ],
	[  9,	"Software overide of {H|V}SYNC <0>." ],
	[ 10,	"Software overide of HSYNC <0>." ],
	[ 11,	"Software overide of VSYNC <0>." ],
	[ 12,	"Software blanking enable <1>." ],
	[ 13,	"Drive video timing Data output <0>." ],
	[ 14,	"Drive video timing Blank output <0>. *" ],
	[ 15,	"Drive video timing {h/v}Sync output <0>. *" ],
	[ 16,	"Drive video timing dclk output <0>. *" ],
	[ 17,	"vclk select (0=vid_clk_2x, 1=vid_clk_slave) <0>" ],
	[ 19,	"vid_clk_2x delay (0=0ns, 1=4ns, 2=6ns, 3=8ns) <0>." ],
	[ 21,	"vclk source : vid_clk(0=_slave, 1=_2x/2, 23=_2x_sel) <2>." ],
	[ 22,	"Enable 24 bpp video output <0>. *" ],
	[ 23,	"Enable SLI (Defalut 0)." ],
	[ 24,	"_v2 num tiles 32x32[5]" ],
	[ 25,	"Enable video edge detection filtering <0>. *" ],
	[ 26,	"Invert vid_clk_2x <0>." ],
	[ 28,	"vid_clk_2x_sel delay (0=0ns, 1=4ns, 2=6ns, 3=8ns) <0>." ],
	[ 30,	"vid_clk_sel delay (0=0ns, 1=4ns, 2=6ns, 3 8ns) <0>." ],
	[ 31,	"Disable fast Rd-ahead-Wr to Rd-ahead-Rd turnaround <0>." ]
);

my @fbiinit2 = (
	[  0,	"Enable video dither subtraction <0>. *" ],
	[  1,	"DRAM banking configuration (0=128Kx16, 1=256Kx16)" ],
	[  3,	"reserved" ],
	[  4,	"Triple Buffering Enable <0>." ],
	[  5,	"Enable fast RAS read cycles <0>." ],
	[  6,	"Enable generated dram OE signal <1>." ],
	[  7,	"Enable fast Rd-Ahead-Wr turnaround (needs bit[6]=1) <0>." ],
	[  8,	"Enable pass-through dither mode (8 BPP) <0>." ],
	[ 10,	"swapbuff algo (0=dac_vsync, 1=dac_data(0), 2=pci_fifo) <0>" ],
	[ 19,	"Video Buffer Offset (640x480 = 150, 832x608 = 297) <0>. *" ],
	[ 20,	"Enable DRAM banking <0>." ],
	[ 21,	"Enable DRAM Read Ahead FIFO <0>." ],
	[ 22,	"DRAM Refresh Enable <0>." ],
	[ 31,	"DRAM Refresh_Load Value ( <<5 ) <0x100>" ]
);

my @fbiinit3 =(
	[  0,	"Triangle register address remap <0>" ],
	[  5,	"Video FIFO threshold <0>" ],
	[  6,	"Disable texture mapping <0>" ],
	[  7,	"" ],
	[ 10,	"FBI mem type (0=EDO, 1 SDRAM) (ro) V2=??" ],
	[ 11,	"VGA PASS reset value (ro)" ],
	[ 12,	"Hardcode PCI base add 0x1f000000" ],
	[ 16,	"fbi-to-trex bus clock delay selections <2>" ],
	[ 21,	"trex-to-fbi bus fifio full threshold <0xf>" ],
	[ 31,	"Y origin Swap substraction value <0>. *" ]
);

my @fbiinit4 = (
	[  0,	"Wait state for PCI read access (0=1ws, 1=2ws) <1>. *" ],
	[  1,	"enable read-ahead logic for LFB reads <0>. *" ],
	[  7,	"Memory FIFO low watermark for PCI fifo <0>." ],
	[ 17,	"Memory FIFO row start <0>." ],
	[ 27,	"Memory FIFO row rollover <0>." ],
	[ 31,	"_v2 video clocking delay control" ]
);

my @fbiinit5 = (
	[  0,	"disable pci stop <?>" ],
	[  1,	"PCI slave is 66Mhz capable (0=33MHz, 1=66) ro <?>" ],
	[  2,	"dac_data output width (0=16bits, 1=24bits) ro <?>" ],
	[  3,	"dac_data[17]/GPIO_0 output value <?>" ],
	[  4,	"dac_data[18]/GPIO_1 control <?>" ],
	[  8,	"generic power-on strapping pins ro <?>" ],
	[ 10,	"Color alloc (0=2col1aux, 1=3col0aux, 2=3col1aux) <0>" ],
	[ 11,	"drive vid_clk_slave output <0>" ],
	[ 12,	"drive data_data[16]" ],
	[ 13,	"vclk input select (0=cf init1[17] 1=dac_data[16] <0>" ],
	[ 14,	"v2 SLI detect <?>" ],
	[ 15,	"sync reads from {h|v}retrace reg <0>" ],
	[ 16,	"horizontal border color enable, right edge <0>" ],
	[ 17,	"horizontal border color enable, left edge <0>" ],
	[ 18,	"vertical border color enable, bottom edge <0>" ],
	[ 19,	"vertical border color enable, top edge <0>" ],
	[ 20,	"scan double video out in Horizontal dim <0>" ],
	[ 21,	"scan double video out in vertical dim <0>. *" ],
	[ 22,	"enable gamma correction for 16bit video output <0>" ],
	[ 23,	"invert dac_hsync output (0=active low, 1=active high) <0>. *" ],
	[ 24,	"invert dac_vsync output (0=active low, 1=active high) <0>. *" ],
	[ 25,	"enable full 24 bit dac_data output <0> conflict/init5" ],
	[ 26,	"interlaced video output <0>. *" ],
	[ 27,	"dact_data[18]/GPIO_1 control (+ init5[4] <0>" ],
	[ 29,	"" ],
	[ 31,	"triangle rasterization mode control <0>" ]
);

my @fbiinit6 = (
	[  2,	"video window active counter (swp algo init2[10:9]=1/2) <0>" ],
	[  7,	"video window drag counter (same as above) <0> " ],
	[  8,	"scanline interleave sync master (swp algo=3) <0>" ],
	[ 10,	"dat_data[22]/GPIO_2 output val (0/1=3state, 2=drive0 3=drive1)" ],
	[ 12,	"dac_data[23]/GPIO_3 output val (0/1=3state, 2=drive0 3=drive1)" ],
	[ 14,	"sli_synci output val (0/1=3state, 2=drive0 3=drive1)" ],
	[ 16,	"sli_synout output val (0=intern signl 1=3state 2=drv0 3=drv1)" ],
	[ 18,	"dac_rd output val (same, 0=internal dac_rd signal)" ],
	[ 20,	"dac_wr output val (same, 0=internal dac_wr signal)" ],
	[ 27,	"PCI FIFO empty entry low watermark ->pcif_rdy dac[21] <0>" ],
	[ 29,	"vga_pass_n output value (0,1=internal, 1=drive0, 3=drive1)" ],
	[ 30,	"num of tiles 32x32[0]" ],
	[ 31,	"" ]
);

my @fbiinit7 = (
	[  7,	"generic power on strapping pins ro <?>" ],
	[  8,	"CMDFIFO enable (exclude mem FIFO)<0>" ],
	[  9,	"CMDFIFO (0=internal fifo, 1= offscreen memory) <0>" ],
	[ 10,	"disable CMDFIFO hole counting logic <0>" ],
	[ 15,	"CMDFIFO read fetch threshold <0>" ],
	[ 16,	"sync writes to CMDFIFO across graphic clock boundary <0>" ],
	[ 17,	"sync reads from CMDFIFO across graphic clock boundry <0>" ],
	[ 18,	"reset PCI packer <0>" ],
	[ 19,	"enable chroma{key|range} writes to 'bruce'<0>" ],
	[ 26,	"CMDFIFO PCI timeout counter <0>" ],
	[ 27,	"enable bursting consecutive texture writes across FT bus <0>" ], 
	[ 31,	"" ]
);

my @initenable = (
	[  0,	"enable write to Hardware init reg <0>" ],
	[  1,	"enable writes to PCI FIFO <0>" ],
	[  2,	"remap fbiinit{2|3} to dac_read, videochecksum <0>" ],
	[  3,	"" ],
	[  4,	"FBI snooping reg 0 enable <0>" ],
	[  5,	"FBI snooping reg 0 mem matching type (0=mem, 1=io) <0>" ],
	[  6,	"FBI snooping reg 0 Rd/Wr matching type (0=Wr, 1=Rd) <0>" ],
	[  7,	"FBI snooping reg 1 enable <0>" ],
	[  8,	"FBI snooping reg 1 mem matching type (0=mem, 1=io) <0>" ],
	[  9,	"FBI snooping reg 1 Rd/Wr matching type (0=Wr, 1=Rd) <0>" ],
	[ 10,	"sli PCI bus owner (0=master, 1=slave)" ],
	[ 11,	"sli master/slave det (0=master/even 1=slave/odd) <0>" ],
	[ 15,	"_ 2nd v2 revision" ],
	[ 19,	"_ v2 manufacturing fab id" ],
	[ 20,	"PCI int enable <?>" ],
	[ 21,	"PCI in timeout enable <0>" ],
	[ 22,	"NAND test tree enable <0>" ],
	[ 23,	"SLI address snoop enable <0>" ],
	[ 31,	"SLI snoop address" ],
);

#	[ ,	"" ],

my %regs = ( "fbiinit0" =>  \@fbiinit0,
	"fbiinit1" =>  \@fbiinit1,
	"fbiinit2" =>  \@fbiinit2,
	"fbiinit3" =>  \@fbiinit3,
	"fbiinit4" =>  \@fbiinit4,
	"fbiinit5" =>  \@fbiinit5,
	"fbiinit6" =>  \@fbiinit6,
	"fbiinit7" =>  \@fbiinit7,
	"initenable" => \@initenable,
	"init" => \@initenable
);

if ($#vals == -1 ) {
	usage();
	exit -1;
}
if ( ! exists $regs{"$reg_name"} ) {
	if (exists $regs{"fbiinit$reg_name"} ) {
		$reg_name = "fbiinit$reg_name";
	} elsif (exists $regs{"fbi$reg_name"} ) {
		$reg_name = "fbi$reg_name";
	} else {
		usage();
		exit (-1);
	}
}


my $reg_val;
print "# $reg_name\n";
print "#\t";
for $reg_val (@vals) {
	if ($reg_val =~ s/^0x//i) {
		$reg_val = hex ($reg_val);
	}
	printf ("%04x\t", ($reg_val>>16) & 0xffff);
}
print "\n";
print "#bit\t";
for $reg_val (@vals) {
	printf ("%04x\t", $reg_val & 0xFFFF );
}
print "desc\n";
my ($val, $bitfield, $bitsize, $field_desc, $desc , $endbit);
my $bit = 0;

for $field_desc (@{$regs{"$reg_name"}}) {
	($endbit, $desc) = @{$field_desc};
	$bitsize = $endbit - $bit +1 ;
	$bitfield = $bit ;
	if ($bitsize != 1 ) {
		$bitfield = "$endbit:$bitfield";
	}
	$desc = "Reserved" if ($desc eq "") ;

	print "$bitfield\t";
	foreach $reg_val (@vals) {
		$val = ($reg_val >> $bit)  & ((1<<$bitsize)-1) ;
		$val ='.' if $val == 0;
		print "$val\t";
	}
	print "$desc\n";
	$bit = $endbit +1;
}
print "\n";


sub usage
{
	print "$0 reg_name value\n";
	print "reg_name in ". join (', ' , keys (%regs ))."\n";
}
